`timescale 1ns / 1ps
//////////////////////////////////////////////////////////////////////////////////
// Company: 
// Engineer: 
// 
// Create Date:    14:37:17 01/27/2011 
// Design Name: 
// Module Name:    vga_disp 
// Project Name: 
// Target Devices: 
// Tool versions: 
// Description: 
//
// Dependencies: 
//
// Revision: 
// Revision 0.01 - File Created
// Additional Comments: 
//
//////////////////////////////////////////////////////////////////////////////////
module vga_graphics(
clk,
rst,
horz_sync,
vert_sync,
rgb
);

input						clk,rst;

output					horz_sync,vert_sync;
output	[2:0]			rgb;

reg		[2:0]			rgb_reg;
wire						signal, pixel_tick;
wire		[9:0] 		pixel_x, pixel_y;
wire		[2:0]			rgb_next;


vga_sync vga1(.clk(clk),.rst(rst),.horz_sync(horz_sync),.vert_sync(vert_sync),
					.signal(signal),.tick(pixel_tick),.x_pix(pixel_x),.y_pix(pixel_y));
					
object obj1(.signal(signal), .pix_x(pixel_x), .pix_y(pixel_y),
						.graph_rgb(rgb_next));
always @ (posedge clk)
begin
	if (pixel_tick)
		rgb_reg <= rgb_next;

end

assign rgb = rgb_reg;

endmodule


module vga_sync(

clk,			//master clock on FPGA board
//clk25MHz,
rst,
//horz_cnt,		//horizontal count
//vert_cnt,		//vertical count
//int_horz_cnt,		//horizontal count
//int_vert_cnt,		//vertical count
horz_sync,		//horizontal syncronize signal
vert_sync,		//vertical synchronize signal
//red_out,		//RED output signal
//green_out,		//GREEN output signal
//blue_out		//BLUE output signal
signal,
tick,
x_pix,
y_pix
);

parameter PAL=10'b10100_00000;	//640;	pixels/active line
parameter LAF=10'b01111_00000; 	//480 lines/active frame
parameter PLD=10'b11000_11111;	//800 pixel/line divider
parameter LFD=10'b10000_01000;	//521 line/frame divider
parameter HPW=10'b00011_00000;	//96 horizontal sync pulse width (pixels)
parameter HFP=10'b11000-00000;	//46 horizontal sync front porch
parameter HBP=10'b00000_10000;	//16	horizontal sync back porch
parameter VPW=10'b00000_00010;	//2 vertical pulse width
parameter VFP=10'b00000_01010;	//10 vertical sync front porch
parameter VBP=10'b00001_00001;	//33 vertical syn back porch

input 					clk,rst;

//output	[2:0]			red_out;
//output	[2:0]			green_out;
//output	[2:1]			blue_out;
//output	[9:0]			horz_cnt,vert_cnt;

output	[9:0]			x_pix,y_pix;
output					horz_sync,vert_sync,tick,signal;
//output					clk25MHz;

//reg						clk25MHz;
//reg		[2:0]			red_out;
//reg		[2:0]			green_out;
//reg		[2:1]			blue_out;
//reg		[9:0]			horz_cnt;
//reg		[9:0]			vert_cnt;

//reg      [7:0]       image [0:8100];
reg		[9:0]			horz_cnt_reg,horz_cnt_nxt;
reg	 	[9:0]			vert_cnt_reg,vert_cnt_nxt;
reg 		 				horz_sync_reg, vert_sync_reg;

wire						mod2_nxt;
wire 		 				horz_sync_nxt, vert_sync_nxt;
wire						horz_end,vert_end,pix_tick;
		
reg mod2_reg;


always @ (posedge clk, posedge rst)
	if (rst) begin
		mod2_reg<=1'b0;
		vert_cnt_reg<=0;
		horz_cnt_reg<=0;
		vert_sync_reg<=1'b0;
		horz_sync_reg<=1'b0;end
	else begin
		mod2_reg<=mod2_nxt;
		vert_cnt_reg<=vert_cnt_nxt;
		horz_cnt_reg<=horz_cnt_nxt;
		vert_sync_reg<=vert_sync_nxt;
		horz_sync_reg<=horz_sync_nxt;end
		
assign mod2_nxt=~mod2_reg;
assign pix_tick=mod2_reg;

assign horz_end=(horz_cnt_reg==(PAL+HFP+HBP+HPW-1));
assign vert_end=(vert_cnt_reg==(LAF+VFP+VBP+VPW-1));

always @ *
begin
	if (pix_tick)
		if (horz_end)
			horz_cnt_nxt=0;
		else
			horz_cnt_nxt=horz_cnt_reg+1;
	else
		horz_cnt_nxt=horz_cnt_reg;
end
		
always @ *
begin
	if (pix_tick & horz_end)
		if(vert_end)
			vert_cnt_nxt=0;
		else
			vert_cnt_nxt=vert_cnt_reg+1;
	else
			vert_cnt_nxt=vert_cnt_reg;
end

assign horz_sync_nxt=(horz_cnt_reg>=(PAL+HBP) && horz_cnt_reg<=(PAL+HBP+HPW-1));
assign vert_sync_nxt=(vert_cnt_reg>=(LAF+VBP) && vert_cnt_reg<=(LAF+VBP+VPW-1));
assign signal=(horz_cnt_reg<PAL)&&(vert_cnt_reg<LAF);		

assign horz_sync=horz_sync_reg;
assign vert_sync=vert_sync_reg;
assign x_pix=horz_cnt_reg;
assign y_pix=vert_cnt_reg;
assign tick=pix_tick;
endmodule

module object(
input wire signal,
input wire [9:0] pix_x, pix_y,
output reg [2:0] graph_rgb
);

localparam MAX_X = 640;
localparam MAX_Y = 480;
localparam WALL_X = 32;
localparam WALL_Y = 35;

wire wall_on;
wire [2:0] wall_rgb;

assign wall_on = (WALL_X <= pix_x) && (pix_x <= WALL_X);
assign wall_rgb = 3'b001;

always @*
	if (~signal)	
		graph_rgb = 3'b000;
	else
		if (wall_on)
			graph_rgb = wall_rgb;
	   else
			graph_rgb = 3'b110;
endmodule			
			
			